@@ -13,12 +13,10 @@ shrunk_cov = (1-shrinkage)*cov + shrinkage*structured_estimate.
 # License: BSD 3 clause
 
 # avoid division truncation
-from __future__ import division
 import warnings
 import numpy as np
 
 from .empirical_covariance_ import empirical_covariance, EmpiricalCovariance
-from ..externals.six.moves import xrange
 from ..utils import check_array
 
 
@@ -45,10 +43,9 @@ def shrunk_covariance(emp_cov, shrinkage=0.1):
 
     Notes
     -----
-    The regularized (shrunk) covariance is given by
+    The regularized (shrunk) covariance is given by:
 
-    (1 - shrinkage)*cov
-      + shrinkage*mu*np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
 
@@ -74,10 +71,10 @@ class ShrunkCovariance(EmpiricalCovariance):
         Specify if the estimated precision is stored
 
     assume_centered : boolean, default False
-        If True, data are not centered before computation.
+        If True, data will not be centered before computation.
         Useful when working with data whose mean is almost, but not exactly
         zero.
-        If False, data are centered before computation.
+        If False, data will be centered before computation.
 
     shrinkage : float, 0 <= shrinkage <= 1, default 0.1
         Coefficient in the convex combination used for the computation
@@ -85,6 +82,9 @@ class ShrunkCovariance(EmpiricalCovariance):
 
     Attributes
     ----------
+    location_ : array-like, shape (n_features,)
+        Estimated location, i.e. the estimated mean.
+
     covariance_ : array-like, shape (n_features, n_features)
         Estimated covariance matrix
 
@@ -96,20 +96,37 @@ class ShrunkCovariance(EmpiricalCovariance):
         Coefficient in the convex combination used for the computation
         of the shrunk estimate.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.covariance import ShrunkCovariance
+    >>> from sklearn.datasets import make_gaussian_quantiles
+    >>> real_cov = np.array([[.8, .3],
+    ...                      [.3, .4]])
+    >>> rng = np.random.RandomState(0)
+    >>> X = rng.multivariate_normal(mean=[0, 0],
+    ...                                   cov=real_cov,
+    ...                                   size=500)
+    >>> cov = ShrunkCovariance().fit(X)
+    >>> cov.covariance_ # doctest: +ELLIPSIS
+    array([[0.7387..., 0.2536...],
+           [0.2536..., 0.4110...]])
+    >>> cov.location_
+    array([0.0622..., 0.0193...])
+
     Notes
     -----
-    The regularized covariance is given by
+    The regularized covariance is given by:
 
-    (1 - shrinkage)*cov
-      + shrinkage*mu*np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
 
     """
     def __init__(self, store_precision=True, assume_centered=False,
                  shrinkage=0.1):
-        super(ShrunkCovariance, self).__init__(store_precision=store_precision,
-                                               assume_centered=assume_centered)
+        super().__init__(store_precision=store_precision,
+                         assume_centered=assume_centered)
         self.shrinkage = shrinkage
 
     def fit(self, X, y=None):
@@ -122,12 +139,12 @@ class ShrunkCovariance(EmpiricalCovariance):
             Training data, where n_samples is the number of samples
             and n_features is the number of features.
 
-        y : not used, present for API consistence purpose.
+        y
+            not used, present for API consistence purpose.
 
         Returns
         -------
         self : object
-            Returns self.
 
         """
         X = check_array(X)
@@ -157,11 +174,11 @@ def ledoit_wolf_shrinkage(X, assume_centered=False, block_size=1000):
     X : array-like, shape (n_samples, n_features)
         Data from which to compute the Ledoit-Wolf shrunk covariance shrinkage.
 
-    assume_centered : Boolean
-        If True, data are not centered before computation.
+    assume_centered : bool
+        If True, data will not be centered before computation.
         Useful to work with data whose mean is significantly equal to
         zero but is not exactly zero.
-        If False, data are centered before computation.
+        If False, data will be centered before computation.
 
     block_size : int
         Size of the blocks into which the covariance matrix will be split.
@@ -176,8 +193,7 @@ def ledoit_wolf_shrinkage(X, assume_centered=False, block_size=1000):
     -----
     The regularized (shrunk) covariance is:
 
-    (1 - shrinkage)*cov
-      + shrinkage * mu * np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
 
@@ -209,8 +225,8 @@ def ledoit_wolf_shrinkage(X, assume_centered=False, block_size=1000):
     beta_ = 0.  # sum of the coefficients of <X2.T, X2>
     delta_ = 0.  # sum of the *squared* coefficients of <X.T, X>
     # starting block computation
-    for i in xrange(n_splits):
-        for j in xrange(n_splits):
+    for i in range(n_splits):
+        for j in range(n_splits):
             rows = slice(block_size * i, block_size * (i + 1))
             cols = slice(block_size * j, block_size * (j + 1))
             beta_ += np.sum(np.dot(X2.T[rows], X2[:, cols]))
@@ -219,7 +235,7 @@ def ledoit_wolf_shrinkage(X, assume_centered=False, block_size=1000):
         beta_ += np.sum(np.dot(X2.T[rows], X2[:, block_size * n_splits:]))
         delta_ += np.sum(
             np.dot(X.T[rows], X[:, block_size * n_splits:]) ** 2)
-    for j in xrange(n_splits):
+    for j in range(n_splits):
         cols = slice(block_size * j, block_size * (j + 1))
         beta_ += np.sum(np.dot(X2.T[block_size * n_splits:], X2[:, cols]))
         delta_ += np.sum(
@@ -254,10 +270,10 @@ def ledoit_wolf(X, assume_centered=False, block_size=1000):
         Data from which to compute the covariance estimate
 
     assume_centered : boolean, default=False
-        If True, data are not centered before computation.
+        If True, data will not be centered before computation.
         Useful to work with data whose mean is significantly equal to
         zero but is not exactly zero.
-        If False, data are centered before computation.
+        If False, data will be centered before computation.
 
     block_size : int, default=1000
         Size of the blocks into which the covariance matrix will be split.
@@ -276,8 +292,7 @@ def ledoit_wolf(X, assume_centered=False, block_size=1000):
     -----
     The regularized (shrunk) covariance is:
 
-    (1 - shrinkage)*cov
-      + shrinkage * mu * np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
 
@@ -292,10 +307,9 @@ def ledoit_wolf(X, assume_centered=False, block_size=1000):
         X = np.reshape(X, (1, -1))
         warnings.warn("Only one sample available. "
                       "You may want to reshape your data array")
-        n_samples = 1
         n_features = X.size
     else:
-        n_samples, n_features = X.shape
+        _, n_features = X.shape
 
     # get Ledoit-Wolf shrinkage
     shrinkage = ledoit_wolf_shrinkage(
@@ -325,10 +339,10 @@ class LedoitWolf(EmpiricalCovariance):
         Specify if the estimated precision is stored.
 
     assume_centered : bool, default=False
-        If True, data are not centered before computation.
+        If True, data will not be centered before computation.
         Useful when working with data whose mean is almost, but not exactly
         zero.
-        If False (default), data are centered before computation.
+        If False (default), data will be centered before computation.
 
     block_size : int, default=1000
         Size of the blocks into which the covariance matrix will be split
@@ -337,6 +351,9 @@ class LedoitWolf(EmpiricalCovariance):
 
     Attributes
     ----------
+    location_ : array-like, shape (n_features,)
+        Estimated location, i.e. the estimated mean.
+
     covariance_ : array-like, shape (n_features, n_features)
         Estimated covariance matrix
 
@@ -348,12 +365,28 @@ class LedoitWolf(EmpiricalCovariance):
         Coefficient in the convex combination used for the computation
         of the shrunk estimate.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> from sklearn.covariance import LedoitWolf
+    >>> real_cov = np.array([[.4, .2],
+    ...                      [.2, .8]])
+    >>> np.random.seed(0)
+    >>> X = np.random.multivariate_normal(mean=[0, 0],
+    ...                                   cov=real_cov,
+    ...                                   size=50)
+    >>> cov = LedoitWolf().fit(X)
+    >>> cov.covariance_ # doctest: +ELLIPSIS
+    array([[0.4406..., 0.1616...],
+           [0.1616..., 0.8022...]])
+    >>> cov.location_
+    array([ 0.0595... , -0.0075...])
+
     Notes
     -----
-    The regularised covariance is::
+    The regularised covariance is:
 
-        (1 - shrinkage)*cov
-                + shrinkage*mu*np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
     and shrinkage is given by the Ledoit and Wolf formula (see References)
@@ -367,8 +400,8 @@ class LedoitWolf(EmpiricalCovariance):
     """
     def __init__(self, store_precision=True, assume_centered=False,
                  block_size=1000):
-        super(LedoitWolf, self).__init__(store_precision=store_precision,
-                                         assume_centered=assume_centered)
+        super().__init__(store_precision=store_precision,
+                         assume_centered=assume_centered)
         self.block_size = block_size
 
     def fit(self, X, y=None):
@@ -380,12 +413,12 @@ class LedoitWolf(EmpiricalCovariance):
         X : array-like, shape = [n_samples, n_features]
             Training data, where n_samples is the number of samples
             and n_features is the number of features.
-        y : not used, present for API consistence purpose.
+        y
+            not used, present for API consistence purpose.
 
         Returns
         -------
         self : object
-            Returns self.
 
         """
         # Not calling the parent object to fit, to avoid computing the
@@ -415,10 +448,10 @@ def oas(X, assume_centered=False):
         Data from which to compute the covariance estimate.
 
     assume_centered : boolean
-      If True, data are not centered before computation.
+      If True, data will not be centered before computation.
       Useful to work with data whose mean is significantly equal to
       zero but is not exactly zero.
-      If False, data are centered before computation.
+      If False, data will be centered before computation.
 
     Returns
     -------
@@ -433,15 +466,12 @@ def oas(X, assume_centered=False):
     -----
     The regularised (shrunk) covariance is:
 
-    (1 - shrinkage)*cov
-      + shrinkage * mu * np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
 
-    The formula we used to implement the OAS
-    does not correspond to the one given in the article. It has been taken
-    from the MATLAB program available from the author's webpage
-    (http://tbayes.eecs.umich.edu/yilun/covestimation).
+    The formula we used to implement the OAS is slightly modified compared
+    to the one given in the article. See :class:`OAS` for more details.
 
     """
     X = np.asarray(X)
@@ -484,12 +514,10 @@ class OAS(EmpiricalCovariance):
     Chen et al., IEEE Trans. on Sign. Proc., Volume 58, Issue 10, October 2010.
 
     The formula used here does not correspond to the one given in the
-    article. It has been taken from the Matlab program available from the
-    authors' webpage (http://tbayes.eecs.umich.edu/yilun/covestimation).
-    In the original article, formula (23) states that 2/p is multiplied by 
-    Trace(cov*cov) in both the numerator and denominator, this operation is omitted
-    in the author's MATLAB program because for a large p, the value of 2/p is so 
-    small that it doesn't affect the value of the estimator. 
+    article. In the original article, formula (23) states that 2/p is
+    multiplied by Trace(cov*cov) in both the numerator and denominator, but
+    this operation is omitted because for a large p, the value of 2/p is
+    so small that it doesn't affect the value of the estimator.
 
     Parameters
     ----------
@@ -497,10 +525,10 @@ class OAS(EmpiricalCovariance):
         Specify if the estimated precision is stored.
 
     assume_centered : bool, default=False
-        If True, data are not centered before computation.
+        If True, data will not be centered before computation.
         Useful when working with data whose mean is almost, but not exactly
         zero.
-        If False (default), data are centered before computation.
+        If False (default), data will be centered before computation.
 
     Attributes
     ----------
@@ -517,10 +545,9 @@ class OAS(EmpiricalCovariance):
 
     Notes
     -----
-    The regularised covariance is::
+    The regularised covariance is:
 
-        (1 - shrinkage)*cov
-                + shrinkage*mu*np.identity(n_features)
+    (1 - shrinkage) * cov + shrinkage * mu * np.identity(n_features)
 
     where mu = trace(cov) / n_features
     and shrinkage is given by the OAS formula (see References)
@@ -541,12 +568,12 @@ class OAS(EmpiricalCovariance):
         X : array-like, shape = [n_samples, n_features]
             Training data, where n_samples is the number of samples
             and n_features is the number of features.
-        y : not used, present for API consistence purpose.
+        y
+            not used, present for API consistence purpose.
 
         Returns
         -------
         self : object
-            Returns self.
 
         """
         X = check_array(X)
